
precision mediump float;

uniform sampler2D texHighRes;
uniform sampler2D texGlow;
uniform sampler2D texBackgr;

uniform vec2 uSmokeGlowStr;
uniform vec3 uColorSmoke;
uniform vec3 uColor0;
uniform vec3 uColor1;
uniform vec3 uColor2;
uniform vec3 uCellSizeAndShimmerStr;

varying vec2 varyTexCoord;
varying highp vec2 varyTexCoordBackgr;

void main()
{
#ifdef SHIMMER
	vec2 tempGrad = vec2( texture2D(texHighRes, varyTexCoord + vec2(uCellSizeAndShimmerStr.x, 0.0)).g - texture2D(texHighRes, varyTexCoord - vec2(uCellSizeAndShimmerStr.x, 0.0)).g,
						  texture2D(texHighRes, varyTexCoord + vec2(0.0, uCellSizeAndShimmerStr.y)).g - texture2D(texHighRes, varyTexCoord - vec2(0.0, uCellSizeAndShimmerStr.y)).g );
	tempGrad *= uCellSizeAndShimmerStr.z;

	vec3 backgrColor = texture2D(texBackgr, varyTexCoordBackgr + tempGrad).rgb;
#else
	vec3 backgrColor = texture2D(texBackgr, varyTexCoordBackgr).rgb;
#endif

	vec2 mainTexSample = texture2D(texHighRes, varyTexCoord).rg;

	float opacity = mainTexSample.r;
	
	// smoothstep should be removed, but I have a good working color spectrum and I don't want to mess it up...
	float t = smoothstep(0.0, 1.0, mainTexSample.g);

	
	
	float EDGE_0 = 0.1;
	float EDGE_1 = 0.2;
	float EDGE_2 = 0.5;
	
	vec3 color = vec3(0.0);
	
	vec3 lowestColor = mix(backgrColor, uColorSmoke, uSmokeGlowStr.x);
	
	float glowColorScale = 1.0 - 0.25 * uSmokeGlowStr.y;
	
	if(t < EDGE_0)
	{
		color = lowestColor;
	}
	else if(t < EDGE_1)
	{
		t = (t-EDGE_0) / (EDGE_1-EDGE_0);
		color = mix(lowestColor, uColor0 * glowColorScale, t);
	}
	else if(t < EDGE_2)
	{
		t = (t-EDGE_1) / (EDGE_2-EDGE_1);
		color = glowColorScale * mix(uColor0, uColor1, t);
	}
	else
	{
		t = (t-EDGE_2) / (1.0-EDGE_2);
		color = glowColorScale * mix(uColor1, uColor2, t);
	}
	
	color = mix(backgrColor, color, opacity);
	color += texture2D(texGlow, varyTexCoord).rgb * uSmokeGlowStr.y;
	
	gl_FragColor = vec4(clamp(color, 0.0, 1.0), opacity);
	//gl_FragColor = vec4(texture2D(texGlow, varyTexCoord).rgb, 1.0);
}
